@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2021
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_CASCADED_CONTAINER_H
-#define POLYMAKE_CASCADED_CONTAINER_H
+#pragma once
 
 #include "polymake/internal/iterators.h"
 #include "polymake/internal/modified_containers.h"
@@ -436,7 +435,6 @@ namespace polymake {
    using pm::cascade;
 }
 
-#endif // POLYMAKE_CASCADED_CONTAINER_H
 
 // Local Variables:
 // mode:C++
