@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2019
+/* Copyright (c) 1997-2021
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
    --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_META_LIST_H
-#define POLYMAKE_META_LIST_H
+#pragma once
 
 #include <type_traits>
 #include "polymake/meta_function.h"
@@ -121,7 +120,7 @@ struct mlist_length
 
 template <typename... Elements>
 struct mlist_length<mlist<Elements...>>
-   : int_constant<sizeof...(Elements)> {};
+   : int_constant<int(sizeof...(Elements))> {};
 
 template <typename T>
 using mlist_is_empty
@@ -147,6 +146,10 @@ template <typename T, typename... Tail, int Pos>
 struct mlist_at<mlist<T, Tail...>, Pos>
    : mlist_at<mlist<Tail...>, Pos-1> {};
 
+/// counting position backwards
+template <typename T, int Pos>
+using mlist_at_rev = mlist_at<T, mlist_length<T>::value-Pos-1>;
+
 
 template <typename T1, typename T2>
 struct mlist_concat2;
@@ -204,7 +207,7 @@ struct mlist_prepend_if<false, Prefix, List>
  *              If omitted, the selected subset stretches up to the end of the source meta-list.
  *  Positions are counted starting with 0.
  */
-template <typename List, int Start, int End=mlist_length<List>::value, int TotalSize=mlist_length<List>::value>
+template <typename List, int Start, int End = mlist_length<List>::value, int TotalSize = mlist_length<List>::value>
 struct mlist_slice
    : mlist_slice<typename mlist_wrap<List>::type, Start, End, TotalSize> {};
 
@@ -241,6 +244,36 @@ struct mlist_slice<mlist<Elements...>, Start, End, TotalSize>
   static_assert(End > 0 && TotalSize > 0, "mlist_slice - invalid end index");
 };
 
+
+/// Extract elements at even positions
+template <typename List>
+struct mlist_even_subset
+   : mlist_even_subset<typename mlist_wrap<List>::type> {};
+
+template <typename T1, typename T2, typename... Tail>
+struct mlist_even_subset<mlist<T1, T2, Tail...>>
+   : mlist_concat<T1, typename mlist_even_subset<mlist<Tail...>>::type> {};
+
+template <>
+struct mlist_even_subset<mlist<>> {
+   using type = mlist<>;
+};
+
+/// Extract elements at odd positions
+template <typename List>
+struct mlist_odd_subset
+   : mlist_odd_subset<typename mlist_wrap<List>::type> {};
+
+template <typename T1, typename T2, typename... Tail>
+struct mlist_odd_subset<mlist<T1, T2, Tail...>>
+   : mlist_concat<T2, typename mlist_odd_subset<mlist<Tail...>>::type> {};
+
+template <>
+struct mlist_odd_subset<mlist<>> {
+   using type = mlist<>;
+};
+
+
 /// Check whether one list coincides with the tail of another list
 template <typename List1, typename List2, bool valid=(mlist_length<List2>::value >= mlist_length<List1>::value)>
 struct mlist_is_tail_of : std::false_type {};
@@ -280,9 +313,8 @@ using mlist_remove_at
 
 
 /// Construct a meta-list consisting of N copies of the same element
-template <typename T, int N>
+template <typename T, size_t N>
 struct mreplicate {
-   static_assert(N>0, "invalid size");
    using type = typename mlist_concat2< typename mreplicate<T, N/2>::type,
                                         typename mreplicate<T, N-N/2>::type >::type;
 };
@@ -747,6 +779,7 @@ struct mlist_match_all<List1, mlist<>, Compare> {
    using complement2 = mlist<>;
 };
 
+
 /***********************************************************************
  *
  *  Tagged meta-lists
@@ -956,7 +989,6 @@ struct mlist_sort<mlist<>, Less> {
 
 }
 
-#endif // POLYMAKE_META_LIST_H
 
 // Local Variables:
 // mode:C++
