@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2019
+/* Copyright (c) 1997-2021
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_TOPAZ_MERGE_VERTICES_H
-#define POLYMAKE_TOPAZ_MERGE_VERTICES_H
+#pragma once
 
 #include "polymake/Array.h"
 #include "polymake/Set.h"
@@ -28,7 +27,7 @@ namespace polymake { namespace topaz {
 
 /** Merges two vertex sets according to their labels.
  *  Afterwards the first set of labels contains all new
- *  labels and the client returns a Map<int,int> mapping 
+ *  labels and the client returns a Map<Int, Int> mapping 
  *  the vertices of the old complex to their new indices.
  *
  *  The corresponding vertices of L1 and L2 are expected
@@ -36,13 +35,12 @@ namespace polymake { namespace topaz {
  *  consecutively starting with 0.
  */
 
-hash_map<int,int> merge_vertices (Array<std::string>& L1, const Array<std::string>& L2);
+hash_map<Int, Int> merge_vertices(Array<std::string>& L1, const Array<std::string>& L2);
 
 void merge_disjoint_vertices (Array<std::string>& L1, const Array<std::string>& L2);
 
 } }
 
-#endif // POLYMAKE_TOPAZ_MERGE_VERTICES_H
 
 // Local Variables:
 // mode:C++
